﻿IF (SELECT TOP (1) 1 FROM SysLic) IS NULL
	RETURN;

IF OBJECT_ID('SysLookupsValues_Backup') IS NOT NULL
	DROP TABLE SysLookupsValues_Backup;

CREATE TABLE SysLookupsValues_Backup (
	Id UNIQUEIDENTIFIER NOT NULL,
	Name NVARCHAR(MAX) NOT NULL,
	TableName NVARCHAR(50),
	CreatedOn DATETIME2(3)
) ON [PRIMARY]

DECLARE @Id UNIQUEIDENTIFIER,
	@Name NVARCHAR(MAX),
	@LczName NVARCHAR(MAX);
DECLARE @cmdDropTrigger NVARCHAR(MAX),
	@cmdDropLczTrigger NVARCHAR(MAX),
	@cmdDropTriggerTemplate NVARCHAR(MAX) =
'IF OBJECT_ID(''CheckValueUpdate{Name}'') IS NOT NULL DROP TRIGGER CheckValueUpdate{Name}';
DECLARE @cmdCreateTrigger NVARCHAR(MAX),
	@cmdCreateLczTrigger NVARCHAR(MAX),
	@cmdCreateTriggerTemplate NVARCHAR(MAX) =
'CREATE TRIGGER CheckValueUpdate{Name}
	 ON {Name}
	 FOR UPDATE
AS
BEGIN
	SET NOCOUNT ON;
	IF UPDATE (Name)
	BEGIN
		INSERT INTO SysLookupsValues_Backup (Id, Name, TableName, CreatedOn)
		SELECT Id, Name, N''{Name}'', GetDate() FROM DELETED;
	END
END';

DECLARE tableNameCursor CURSOR LOCAL FAST_FORWARD FOR
	SELECT DISTINCT
		ss.Id
	   ,ss.Name
	   ,CASE
	    	WHEN ss.Name = 'OpportunityParticipantRole' THEN 'SysOpportunityParticipRoleLcz'
	    	WHEN ss.Name = 'MailboxForIncidentRegistration' THEN 'SysMailboxForIncidentRegisLcz'
	    	WHEN ss.Name = 'SupplyPaymentTemplateItem' THEN 'SysSupplyPaymTemplItemLcz'
	    	WHEN ss.Name = 'EmailTemplateRecipientType' THEN 'SysEmailTemplRecipientTypeLcz'
	    	WHEN ss.Name = 'SynchronizationProcessState' THEN 'SysSyncProcessStateLcz'
	    	WHEN ss.Name = 'GlobalDuplicateSearchStatus' THEN 'SysGlobalDuplSearchStatusLcz'
	    	WHEN ss.Name = 'SysEntitySchemaColRightLevel' THEN 'SysEntitySchColRightLevelLcz'
	    	WHEN ss.Name = 'EntitySchemaRecRightOperation' THEN 'SysEntitySchRecRightOprLcz'
	    	WHEN ss.Name = 'SysEntitySchemaParentAsscType' THEN 'SysEntitySchParentAsscTypeLcz'
	    	WHEN ss.Name = 'SysEntitySchemaRecOprRightLvl' THEN 'SysEntitySchRecOprRightLvlLcz'
	    	WHEN ss.Name = 'SysEntitySchemaRecRightSource' THEN 'SysEntitySchRecRightSourceLcz'
	    	WHEN LEFT(ss.Name, 3) = 'Sys' THEN ss.Name + 'Lcz'
	    	ELSE 'Sys' + ss.Name + 'Lcz'
	    END AS LczName
	FROM SysSchema ss
	JOIN Lookup l ON ss.UId = l.SysEntitySchemaUId
	JOIN INFORMATION_SCHEMA.COLUMNS isc ON isc.TABLE_NAME = ss.Name
	WHERE isc.COLUMN_NAME = 'Name'
		AND ss.Name NOT LIKE 'Vw%'
		AND ss.Name <> 'ExcelImportLog'
		AND ss.Name <> 'HeaderProperty';

OPEN tableNameCursor;
FETCH NEXT FROM tableNameCursor INTO @Id, @Name, @LczName
WHILE @@FETCH_STATUS = 0
BEGIN
	SET @cmdDropTrigger = REPLACE(@cmdDropTriggerTemplate, '{Name}', @Name);
	EXEC (@cmdDropTrigger);

	SET @cmdCreateTrigger = REPLACE(@cmdCreateTriggerTemplate, '{Name}', @Name);
	BEGIN TRY
		EXEC (@cmdCreateTrigger);
	END TRY
	BEGIN CATCH
		PRINT ERROR_MESSAGE();
		PRINT 'on table '+ @Name;
	END CATCH

	SET @cmdDropLczTrigger = REPLACE(@cmdDropTriggerTemplate, '{Name}', @LczName);
	EXEC (@cmdDropLczTrigger);

	SET @cmdCreateLczTrigger = REPLACE(@cmdCreateTriggerTemplate, '{Name}', @LczName);
	BEGIN TRY
		EXEC (@cmdCreateLczTrigger);
	END TRY
	BEGIN CATCH
		PRINT ERROR_MESSAGE();
		PRINT 'on table '+ @Name;
	END CATCH
	FETCH NEXT FROM tableNameCursor INTO @Id, @Name,  @LczName;
END
DEALLOCATE tableNameCursor;

IF OBJECT_ID('UpdatedSysSettingsValuesLog') IS NOT NULL
	DROP TABLE UpdatedSysSettingsValuesLog;

CREATE TABLE [dbo].UpdatedSysSettingsValuesLog
(
	[Id] [uniqueidentifier] NOT NULL DEFAULT (NEWID()),
	[SysSettingsId] [uniqueidentifier] NULL,
	[SysAdminUnitId] [uniqueidentifier] NULL,
	[IsDef] [bit] NOT NULL DEFAULT ((0)),
	[TextValue] [nvarchar](MAX) NOT NULL DEFAULT (''),
	[IntegerValue] [int] NOT NULL DEFAULT ((0)),
	[FloatValue] [decimal](18, 2) NOT NULL DEFAULT ((0)),
	[BooleanValue] [bit] NOT NULL DEFAULT ((0)),
	[DateTimeValue] [datetime2](3) NULL,
	[GuidValue] [uniqueidentifier] NULL,
	[BinaryValue] [varbinary](MAX) NULL,
	[CreatedOn] [datetime2](3)
);

IF OBJECT_ID('CheckSysSettingsValueUpdate') IS NOT NULL
	DROP TRIGGER CheckSysSettingsValueUpdate;

DECLARE @cmdSysSettingsCreateTrigger NVARCHAR(MAX) =
';CREATE TRIGGER CheckSysSettingsValueUpdate
	ON SysSettingsValue
	FOR UPDATE
AS 
BEGIN
	SET NOCOUNT ON;
	INSERT INTO UpdatedSysSettingsValuesLog ([Id]
		,[SysSettingsId]
		,[SysAdminUnitId]
		,[IsDef]
		,[TextValue]
		,[IntegerValue]
		,[FloatValue]
		,[BooleanValue]
		,[DateTimeValue]
		,[GuidValue]
		,[BinaryValue]
		,[CreatedOn])
	SELECT [Id]
		,[SysSettingsId]
		,[SysAdminUnitId]
		,[IsDef]
		,[TextValue]
		,[IntegerValue]
		,[FloatValue]
		,[BooleanValue]
		,[DateTimeValue]
		,[GuidValue]
		,[BinaryValue]
		,GETDATE() FROM DELETED
END';
EXEC(@cmdSysSettingsCreateTrigger);
